from netbox.views import generic
from . import forms, models, tables
from django.contrib.contenttypes.models import ContentType
from django.shortcuts import get_object_or_404
from django.http import HttpResponseRedirect


class PTUEventView(generic.ObjectView):
    queryset = models.PTUEvent.objects.all()


class PTUEventListView(generic.ObjectListView):
    queryset = models.PTUEvent.objects.all()
    table = tables.PTUEventTable
    # actions = ('add', 'export')


class PTUEventEditView(generic.ObjectEditView):
    queryset = models.PTUEvent.objects.all()
    form = forms.PTUEventForm


class PTUEventDeleteView(generic.ObjectDeleteView):
    queryset = models.PTUEvent.objects.all()


# PTPTUEvent relation


class PTUEventRelationView(generic.ObjectView):
    queryset = models.PTUEventRelation.objects.all()


class PTUEventRelationListView(generic.ObjectListView):
    queryset = models.PTUEventRelation.objects.all()
    table = tables.PTUEventRelationTable


class PTUEventRelationEditView(generic.ObjectEditView):
    queryset = models.PTUEventRelation.objects.all()
    form = forms.PTUEventRelationForm


class PTUEventRelationDeleteView(generic.ObjectDeleteView):
    queryset = models.PTUEventRelation.objects.all()


# #
# # PTUEvent assignments
# #

class PTUEventAssignmentEditView(generic.ObjectEditView):
    queryset = models.PTUEventAssignment.objects.all()
    form = forms.PTUEventAssignmentForm
    template_name = 'ptuevents/ptuevent_assignment_edit.html'

    def alter_object(self, instance, request, args, kwargs):
        if not instance.pk:
            # Assign the object based on URL kwargs
            content_type = get_object_or_404(
                ContentType, pk=request.GET.get('content_type'))
            instance.object = get_object_or_404(
                content_type.model_class(), pk=request.GET.get('object_id'))
        return instance

    def post(self, request, *args, **kwargs):
        form = forms.PTUEventAssignmentForm(request.POST)
        if form.is_valid():
            content_type_id = request.GET.get('content_type', -1)
            object_id = request.GET.get('object_id', -1)
            ptuevent = form.cleaned_data['ptuevent']
            qs = models.PTUEventAssignment.objects.filter(
                content_type=content_type_id, object_id=object_id, ptuevent=ptuevent.id)
            if qs.exists():
                redirect_url = request.GET.get('return_url', '/')
                return HttpResponseRedirect(redirect_url)

        return super().post(request, *args, **kwargs)


class PTUEventAssignmentDeleteView(generic.ObjectDeleteView):
    queryset = models.PTUEventAssignment.objects.all()


class PTAppSystemView(generic.ObjectView):
    queryset = models.PTAppSystem.objects.all()

    def get_extra_context(self, request, instance):
        print(self)
        print(request)
        print(instance.id)
        app_system_assignments = models.PTAppSystemAssignment.objects.filter(
            app_system=instance)
        assignments_table = tables.AppSystemAssignmentTable(
            app_system_assignments)
        assignments_table.columns.hide('app_system')
        assignments_table.configure(request)
        content_type_id = ContentType.objects.get_for_model(
            model=models.PTAppSystem).id
        PTUEvent_ass = models.PTUEventAssignment.objects.filter(
            object_id=instance.id, content_type=content_type_id)
        PTUEvents = []
        for r in PTUEvent_ass:
            PTUEvents.append({
                'assignment_id': r.id,
                'name': r.ptuevent,
                'rel': r.relation.name
            })

        return {
            'assignments_table': assignments_table,
            'PTUEvents': PTUEvents
        }


class PTAppSystemListView(generic.ObjectListView):
    queryset = models.PTAppSystem.objects.all()
    table = tables.AppSystemTable


class PTAppSystemEditView(generic.ObjectEditView):
    queryset = models.PTAppSystem.objects.all()
    form = forms.PTAppSystemForm


class PTAppSystemDeleteView(generic.ObjectDeleteView):
    queryset = models.PTAppSystem.objects.all()


class PTAppSystemAssignmentEditView(generic.ObjectEditView):
    queryset = models.PTAppSystemAssignment.objects.all()
    form = forms.PTAppSystemAssignmentForm
    template_name = 'ptuevents/appsystem_assignment_edit.html'

    def alter_object(self, instance, request, args, kwargs):
        if not instance.pk:
            # Assign the object based on URL kwargs
            content_type = get_object_or_404(
                ContentType, pk=request.GET.get('content_type'))
            instance.object = get_object_or_404(
                content_type.model_class(), pk=request.GET.get('object_id'))
        return instance

    def get_extra_addanother_params(self, request):
        return {
            'content_type': request.GET.get('content_type'),
            'object_id': request.GET.get('object_id'),
        }

    def post(self, request, *args, **kwargs):
        form = forms.PTAppSystemAssignmentForm(request.POST)
        if form.is_valid():
            content_type_id = request.GET.get('content_type', -1)
            object_id = request.GET.get('object_id', -1)
            s = form.cleaned_data['app_system']
            qs = models.PTAppSystemAssignment.objects.filter(
                content_type=content_type_id, object_id=object_id, app_system=s.id)
            if qs.exists():
                redirect_url = request.GET.get('return_url', '/')
                return HttpResponseRedirect(redirect_url)

        return super().post(request, *args, **kwargs)


class PTAppSystemAssignmentDeleteView(generic.ObjectDeleteView):
    queryset = models.PTAppSystemAssignment.objects.all()


# PTUsers
class PTUsersView(generic.ObjectView):
    queryset = models.PTUsers.objects.all()
    template_name = 'ptuevents/ptusers_add.html'


class PTUsersListView(generic.ObjectListView):
    queryset = models.PTUsers.objects.all()
    table = tables.PTUsersTable


class PTUsersEditView(generic.ObjectEditView):
    queryset = models.PTUsers.objects.all()
    form = forms.PTUsersForm


class PTUsersDeleteView(generic.ObjectDeleteView):
    queryset = models.PTUsers.objects.all()


# PTWorkstations

class PTWorkstationsView(generic.ObjectView):
    queryset = models.PTWorkstations.objects.all()


class PTWorkstationsListView(generic.ObjectListView):
    queryset = models.PTWorkstations.objects.all()
    table = tables.PTWorkstationsTable


class PTWorkstationsEditView(generic.ObjectEditView):
    queryset = models.PTWorkstations.objects.all()
    form = forms.PTWorkstationsForm


class PTWorkstationsDeleteView(generic.ObjectDeleteView):
    queryset = models.PTWorkstations.objects.all()



from netbox.forms import NetBoxModelForm
from tenancy.models import Tenant
from pyrsistent import v
from .models import PTUEvent, PTUEventRelation, PTUEventAssignment, PTAppSystem, PTAppSystemAssignment, PTUsers, PTWorkstations
from utilities.forms.fields import CommentField
from utilities.forms import (
    BootstrapMixin, DynamicModelChoiceField)
from django import forms


class PTUEventForm(NetBoxModelForm):
    class Meta:
        model = PTUEvent
        fields = ('name', 'description', 'comments')


class PTUEventRelationForm(NetBoxModelForm):
    class Meta:
        model = PTUEventRelation
        fields = ('name', 'description')


class PTUEventAssignmentForm(BootstrapMixin, forms.ModelForm):
    ptuevent = DynamicModelChoiceField(
        queryset=PTUEvent.objects.all()
    )
    relation = DynamicModelChoiceField(
        queryset=PTUEventRelation.objects.all()
    )

    class Meta:
        model = PTUEventAssignment
        fields = (
            'ptuevent', 'relation',
        )


class PTAppSystemForm(NetBoxModelForm):
    comments = CommentField()

    class Meta:
        model = PTAppSystem
        fields = ('name', 'slug', 'tenant', 'description', 'comments', 'tags')


class PTAppSystemAssignmentForm(BootstrapMixin, forms.ModelForm):
    tenant = DynamicModelChoiceField(
        queryset=Tenant.objects.all(), required=True)
    app_system = DynamicModelChoiceField(queryset=PTAppSystem.objects.all(), query_params={
        'tenant': '$tenant',
    })

    class Meta:
        model = PTAppSystemAssignment
        fields = ('app_system',)


class PTUsersForm(NetBoxModelForm):
    class Meta:
        model = PTUsers
        fields = ('name',
                  'sAMAccountName',
                  'status',
                  'firstname',
                  'lastname', 'ad_guid', 'ad_description', 'position', 'department', 'comment',
                  'vpnIPaddress',
                  'description',
                  'comments')


class PTWorkstationsForm(NetBoxModelForm):
    class Meta:
        model = PTWorkstations
        fields = ('name', 'CN', 'DistinguishedName', 'ad_guid', 'ad_description', 'description')




from django.urls import path
from . import models, views
from netbox.views.generic import ObjectChangeLogView

urlpatterns = (
    # PTUEvents
    path('ptuevents/', views.PTUEventListView.as_view(), name='ptuevent_list'),
    path('ptuevents/add/', views.PTUEventEditView.as_view(), name='ptuevent_add'),
    path('ptuevents/<int:pk>/', views.PTUEventView.as_view(), name='ptuevent'),
    path('ptuevents/<int:pk>/edit/', views.PTUEventEditView.as_view(),
         name='ptuevent_edit'),
    path('ptuevents/<int:pk>/delete/', views.PTUEventDeleteView.as_view(),
         name='ptuevent_delete'),
    path('ptuevents/<int:pk>/changelog/', ObjectChangeLogView.as_view(), name='ptuevent_changelog', kwargs={
        'model': models.PTUEvent
    }),
    #  PTUEvent relation
    path('ptueventrelation/', views.PTUEventRelationListView.as_view(),
         name='ptueventrelation_list'),
    path('ptueventrelation/add/', views.PTUEventRelationEditView.as_view(),
         name='ptueventrelation_add'),
    path('ptueventrelation/<int:pk>/',
         views.PTUEventRelationView.as_view(), name='ptueventrelation'),
    path('ptueventrelation/<int:pk>/edit/',
         views.PTUEventRelationEditView.as_view(), name='ptueventrelation_edit'),
    path('ptueventrelation/<int:pk>/delete/',
         views.PTUEventRelationDeleteView.as_view(), name='ptueventrelation_delete'),
    path('ptueventrelation/<int:pk>/changelog/', ObjectChangeLogView.as_view(), name='ptueventrelation_changelog', kwargs={
        'model': models.PTUEventRelation
    }),
    # PTUEvent assignment
    path('ptueventassignment/add/', views.PTUEventAssignmentEditView.as_view(),
         name='ptueventassignment_add'),
    path('ptueventassignment/<int:pk>/edit/', views.PTUEventAssignmentEditView.as_view(),
         name='ptueventassignment_edit'),
    path('ptueventassignment/<int:pk>/delete/', views.PTUEventAssignmentDeleteView.as_view(),
         name='ptueventassignment_delete'),

    path('app-systems/', views.PTAppSystemListView.as_view(),
         name="ptappsystem_list"),
    path('app-systems/add', views.PTAppSystemEditView.as_view(),
         name="ptappsystem_add"),
    path('app-systems/<int:pk>/',
         views.PTAppSystemView.as_view(), name="ptappsystem"),
    path('app-systems/<int:pk>/edit',
         views.PTAppSystemEditView.as_view(), name="ptappsystem_edit"),
    path('app-systems/<int:pk>/delete',
         views.PTAppSystemDeleteView.as_view(), name="ptappsystem_delete"),
    path('app-systems/<int:pk>/changelog', ObjectChangeLogView.as_view(),
         name="ptappsystem_changelog", kwargs={'model': models.PTAppSystem}),

    # app system assignment
    path('app-system-assignment/add/', views.PTAppSystemAssignmentEditView.as_view(),
         name='ptappsystemassignment_add'),
    path('app-system-assignment/<int:pk>/edit/', views.PTAppSystemAssignmentEditView.as_view(),
         name='ptappsystemassignment_edit'),
    path('app-system-assignment/<int:pk>/delete/', views.PTAppSystemAssignmentDeleteView.as_view(),
         name='ptappsystemassignment_delete'),

    # PTUsers
    path('ptusers/', views.PTUsersListView.as_view(), name='ptusers_list'),
    path('ptusers/add/', views.PTUsersEditView.as_view(), name='ptusers_add'),
    path('ptusers/<int:pk>/', views.PTUsersView.as_view(), name='ptusers'),
    path('ptusers/<int:pk>/edit/', views.PTUsersEditView.as_view(), name='ptusers_edit'),
    path('ptusers/<int:pk>/delete/', views.PTUsersDeleteView.as_view(), name='ptusers_delete'),
    path('ptusers/<int:pk>/changelog/', ObjectChangeLogView.as_view(), name='ptusers_changelog', kwargs={ 'model': models.PTUsers}),

    # PTWorkstations
    path('ptworkstations/', views.PTWorkstationsListView.as_view(), name='ptworkstations_list'),
    path('ptworkstations/add/', views.PTWorkstationsEditView.as_view(), name='ptworkstations_add'),
    path('ptworkstations/<int:pk>/', views.PTWorkstationsView.as_view(), name='ptworkstations'),
    path('ptworkstations/<int:pk>/edit/', views.PTWorkstationsEditView.as_view(), name='ptworkstations_edit'),
    path('ptworkstations/<int:pk>/delete/', views.PTWorkstationsDeleteView.as_view(), name='ptworkstations_delete'),
    path('ptworkstations/<int:pk>/changelog/', ObjectChangeLogView.as_view(), name='ptworkstations_changelog', kwargs={ 'model': models.PTWorkstations}),

)



from extras.plugins import PluginMenuItem
from extras.plugins import PluginMenuButton, PluginMenuItem
from utilities.choices import ButtonColorChoices


PTUEvent_buttons = [
    PluginMenuButton(
        link='plugins:ptuevents:ptuevent_add',
        title='Add',
        icon_class='mdi mdi-plus-thick',
        color=ButtonColorChoices.GREEN
    )
]
PTUEvent_rel_buttons = [
    PluginMenuButton(
        link='plugins:ptuevents:ptueventrelation_add',
        title='Add',
        icon_class='mdi mdi-plus-thick',
        color=ButtonColorChoices.GREEN
    )
]

appsystem_buttons = [
    PluginMenuButton(
        link='plugins:ptuevents:ptappsystem_add',
        title='Add',
        icon_class='mdi mdi-plus-thick',
        color=ButtonColorChoices.GREEN
    )
]

PTUsers_buttons = [
    PluginMenuButton(
        link='plugins:ptuevents:ptusers_add',
        title='Add',
        icon_class='mdi mdi-plus-thick',
        color=ButtonColorChoices.GREEN
    )
]

PTWorkstations_buttons = [
    PluginMenuButton(
        link='plugins:ptuevents:ptworkstations_add',
        title='Add',
        icon_class='mdi mdi-plus-thick',
        color=ButtonColorChoices.GREEN
    )
]

menu_items = (
    PluginMenuItem(
        link='plugins:ptuevents:ptuevent_list',
        link_text='Events',
        buttons=PTUEvent_buttons
    ),
    PluginMenuItem(
        link='plugins:ptuevents:ptueventrelation_list',
        link_text='Event Relations',
        buttons=PTUEvent_rel_buttons
    ),
    PluginMenuItem(
        link='plugins:ptuevents:ptappsystem_list',
        link_text='App Systems',
        buttons=appsystem_buttons
    ),
    PluginMenuItem(
        link='plugins:ptuevents:ptusers_list',
        link_text='Users',
        buttons=PTUsers_buttons
    ),
    PluginMenuItem(
        link='plugins:ptuevents:ptworkstations_list',
        link_text='Workstations',
        buttons=PTWorkstations_buttons
    ),
)



# Generated by Django 4.1.5 on 2023-01-17 13:35

from django.db import migrations, models
import django.db.models.deletion
import taggit.managers

# fix module
# import utilities.json
import django.core.serializers.json


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        # ('tenancy', '0009_standardize_description_comments'),
        ('contenttypes', '0002_remove_content_type_name')  # ,
        # ('extras', '0084_staging'),
    ]

    operations = [
        migrations.CreateModel(
            name='PTAppSystem',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=django.core.serializers.json.DjangoJSONEncoder)),
                ('name', models.CharField(max_length=200)),
                ('slug', models.SlugField(max_length=100, unique=True)),
                ('description', models.CharField(blank=True, max_length=200)),
                ('comments', models.TextField(blank=True)),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
                ('tenant', models.ForeignKey(null=True, on_delete=django.db.models.deletion.PROTECT, related_name='ptappsystem', to='tenancy.tenant')),
            ],
            options={
                'verbose_name_plural': 'Application systems',
                'ordering': ['name'],
            },
        ),
        migrations.CreateModel(
            name='PTUEvent',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=django.core.serializers.json.DjangoJSONEncoder)),
                ('name', models.CharField(max_length=250, unique=True)),
                ('description', models.CharField(blank=True, max_length=500)),
                ('comments', models.TextField(blank=True)),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
            ],
            options={
                'verbose_name_plural': 'Unacceptable events',
                'ordering': ['name'],
            },
        ),
        migrations.CreateModel(
            name='PTUEventRelation',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=django.core.serializers.json.DjangoJSONEncoder)),
                ('name', models.CharField(max_length=100, unique=True)),
                ('description', models.CharField(blank=True, max_length=500)),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
            ],
            options={
                'verbose_name_plural': 'Event relationship',
                'ordering': ['name'],
            },
        ),
        migrations.CreateModel(
            name='PTUEventAssignment',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('object_id', models.PositiveBigIntegerField()),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.contenttype')),
                ('ptuevent', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='ptuevent_assignments', to='ptuevents.ptuevent')),
                ('relation', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='ptuevent_assignments', to='ptuevents.ptueventrelation')),
            ],
            options={
                'ordering': ['ptuevent'],
                'unique_together': {('content_type', 'object_id', 'ptuevent', 'relation')},
            },
        ),
        migrations.CreateModel(
            name='PTAppSystemAssignment',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('object_id', models.PositiveBigIntegerField()),
                ('app_system', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='ptappsystem_assignments', to='ptuevents.ptappsystem')),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.contenttype')),
            ],
            options={
                'ordering': ['app_system'],
                'unique_together': {('content_type', 'object_id', 'app_system')},
            },
        ),
        migrations.CreateModel(
                name='PTUsers',
                fields=[
                    ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                    ('created', models.DateTimeField(auto_now_add=True, null=True)),
                    ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                    ('custom_field_data', models.JSONField(blank=True, default=dict,
                                                           encoder=django.core.serializers.json.DjangoJSONEncoder)),
                    ('name', models.CharField(max_length=250, unique=False)),

                    ('firstname', models.CharField(max_length=250, blank=True)),
                    ('lastname', models.CharField(max_length=250, blank=True)),
                    ('status', models.CharField(max_length=250, unique=False, blank=True, default='enabled')),
                    ('sAMAccountName', models.CharField(max_length=250, unique=True, blank=False)),
                    ('ad_guid', models.CharField(max_length=250, blank=True)),
                    ('vpnIPaddress', models.CharField(max_length=250, blank=True)),
                    ('ad_description', models.CharField(max_length=250, blank=True)),
                    ('position', models.CharField(max_length=250, blank=True)),
                    ('department', models.CharField(max_length=250, blank=True)),
                    ('comment', models.CharField(max_length=250, blank=True)),

                    ('description', models.CharField(blank=True, max_length=500)),
                    ('comments', models.TextField(blank=True)),
                    ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
                ],
                options={
                    'verbose_name_plural': 'Users',
                    'ordering': ['name'],
                },
        ),
        migrations.CreateModel(
                name='PTWorkstations',
                fields=[
                    ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                    ('created', models.DateTimeField(auto_now_add=True, null=True)),
                    ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                    ('custom_field_data', models.JSONField(blank=True, default=dict,
                                                           encoder=django.core.serializers.json.DjangoJSONEncoder)),
                    ('name', models.CharField(max_length=100, unique=True)),
                    ('CN', models.CharField(max_length=250, unique=True)),
                    ('DistinguishedName', models.CharField(max_length=500, unique=True)),
                    ('ad_guid', models.CharField(max_length=250, unique=True)),
                    ('ad_description', models.CharField(max_length=500, unique=True)),
                    ('description', models.CharField(blank=True, max_length=500)),
                    ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
                ],
                options={
                    'verbose_name_plural': 'Workstations',
                    'ordering': ['name'],
                },
        )
    ]



import django_filters
from netbox.filtersets import ChangeLoggedModelFilterSet, NetBoxModelFilterSet
from utilities.filters import ContentTypeFilter
from .models import *
from django.db.models import Q


class PTUEventAssignmentFilterSet(ChangeLoggedModelFilterSet):
    content_type = ContentTypeFilter()
    ptuevent_id = django_filters.ModelMultipleChoiceFilter(
        queryset=PTUEvent.objects.all(),
        label='PTUEvent (ID)',
    )
    relation_id = django_filters.ModelMultipleChoiceFilter(
        queryset=PTUEventRelation.objects.all(),
        label='PTUEvent relation (ID)',
    )
    relation = django_filters.ModelMultipleChoiceFilter(
        field_name='relation__name',
        queryset=PTUEventRelation.objects.all(),
        to_field_name='name',
        label='PTUEvent relation (name)',
    )

    class Meta:
        model = PTUEventAssignment
        fields = ['id', 'content_type_id', 'object_id']


class PTAppSystemFilterSet(NetBoxModelFilterSet):
    class Meta:
        model = PTAppSystem
        fields = ['id', 'name', 'slug', 'tenant', 'description', 'comments']

    def search(self, queryset, name, value):
        if not value.strip():
            return queryset
        return queryset.filter(
            Q(name__icontains=value) |
            Q(tenant__icontains=value) |
            Q(slug__icontains=value) |
            Q(description__icontains=value) |
            Q(comments__icontains=value)
        )


class PTAppSystemAssignmentFilterSet(ChangeLoggedModelFilterSet):
    content_type = ContentTypeFilter()
    app_system_id = django_filters.ModelMultipleChoiceFilter(
        queryset=PTAppSystem.objects.all(),
        label='PTAppSystem (ID)',
    )

    class Meta:
        model = PTAppSystemAssignment
        fields = ['id', 'content_type_id', 'object_id']



from enum import unique
from django.contrib.postgres.fields import ArrayField
from django.db import models
from netbox.models import NetBoxModel, ChangeLoggedModel
from django.urls import reverse
from netbox.models.features import WebhooksMixin
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.db.models.signals import post_delete
from django.dispatch import receiver
from virtualization.models import VirtualMachine


class PTUEvent(NetBoxModel):
    name = models.CharField(
        max_length=250,
        unique=True
    )

    description = models.CharField(
        max_length=500,
        blank=True,
    )
    comments = models.TextField(blank=True)

    class Meta:
        ordering = ['name']
        verbose_name_plural = 'Unacceptable events'

    def __str__(self):
        return self.name

    def get_absolute_url(self):
        return reverse('plugins:ptuevents:ptuevent', args=[self.pk])


class PTUEventRelation(NetBoxModel):
    name = models.CharField(
        max_length=100,
        unique=True
    )

    description = models.CharField(
        max_length=500,
        blank=True,
    )

    class Meta:
        ordering = ['name']
        verbose_name_plural = 'Event relationship'

    def __str__(self):
        # return 'reskrel'
        return self.name

    def get_absolute_url(self):
        return reverse('plugins:ptuevents:ptueventrelation', args=[self.pk])


class PTUEventAssignment(WebhooksMixin, ChangeLoggedModel):
    content_type = models.ForeignKey(
        to=ContentType,
        on_delete=models.CASCADE
    )
    object_id = models.PositiveBigIntegerField()
    object = GenericForeignKey(
        ct_field='content_type',
        fk_field='object_id'
    )
    ptuevent = models.ForeignKey(
        to='ptuevents.ptuevent',
        on_delete=models.PROTECT,
        related_name='ptuevent_assignments'
    )
    relation = models.ForeignKey(
        to='ptuevents.PTUEventRelation',
        on_delete=models.PROTECT,
        related_name='ptuevent_assignments'
    )

    clone_fields = ('content_type', 'object_id')

    class Meta:
        ordering = ['ptuevent']
        unique_together = ('content_type', 'object_id',
                           'ptuevent', 'relation')

    def __str__(self):
        return str(self.ptuevent)

    def get_absolute_url(self):
        return reverse('plugins:ptuevents:ptuevent', args=[self.ptuevent.pk])


@receiver(post_delete, sender=VirtualMachine, dispatch_uid='del_PTUEvent_assignment')
def del_assignments(sender, **kwargs):
    content_type_id = ContentType.objects.get(model='virtualmachine').id
    instance_id = kwargs.get('instance').id
    objs = PTUEventAssignment.objects.filter(
        object_id=instance_id, content_type=content_type_id)
    objs.delete()


class PTAppSystem(NetBoxModel):
    name = models.CharField(max_length=200)
    slug = models.SlugField(max_length=100, unique=True)
    description = models.CharField(max_length=200, blank=True)
    comments = models.TextField(blank=True)
    tenant = models.ForeignKey(
        to='tenancy.Tenant',
        on_delete=models.PROTECT,
        related_name='ptappsystem',
        blank=False,
        null=True
    )

    def __str__(self):
        return self.name

    def get_absolute_url(self):
        return reverse('plugins:ptuevents:ptappsystem', args=[self.pk])

    class Meta:
        ordering = ['name']
        verbose_name_plural = 'Application systems'


class PTAppSystemAssignment(WebhooksMixin, ChangeLoggedModel):
    content_type = models.ForeignKey(
        to=ContentType,
        on_delete=models.CASCADE
    )
    object_id = models.PositiveBigIntegerField()
    object = GenericForeignKey(
        ct_field='content_type',
        fk_field='object_id'
    )
    app_system = models.ForeignKey(
        to='ptuevents.PTAppSystem',
        on_delete=models.PROTECT,
        related_name='ptappsystem_assignments'
    )

    clone_fields = ('content_type', 'object_id')

    class Meta:
        ordering = ['app_system']
        unique_together = ('content_type', 'object_id',
                           'app_system')

    def __str__(self):
        return str(self.app_system)

    def get_absolute_url(self):
        return reverse('plugins:ptuevents:ptappsystem', args=[self.app_system.pk])


@receiver(post_delete, sender=VirtualMachine, dispatch_uid='del_appsystem_assignment')
def del_assignments(sender, **kwargs):
    content_type_id = ContentType.objects.get(model='virtualmachine').id
    instance_id = kwargs.get('instance').id
    objs = PTAppSystemAssignment.objects.filter(
        object_id=instance_id, content_type=content_type_id)
    objs.delete()


class PTUsers(NetBoxModel):
    name = models.CharField(
        max_length=250,
        unique=False
    )

    firstname = models.CharField(
        max_length=250,
        blank=True,
    )
    lastname = models.CharField(
        max_length=250,
        blank=True,
    )

    ENABLED = 'enabled'
    DISABLED = 'disabled'
    DELETED = 'deleted'
    NEEDACTION = 'need action'

    CHOICES = (
        (ENABLED, ENABLED),
        (DISABLED, DISABLED),
        (DELETED, DELETED),
        (NEEDACTION, NEEDACTION),
    )

    status = models.CharField(
        max_length=250,
        unique=False,
        choices=CHOICES,
        default=ENABLED
    )

    sAMAccountName = models.CharField(
        max_length=250,
        unique=True,
        blank=False,
    )

    ad_guid = models.CharField(
        max_length=250,
        blank=True,
    )

    vpnIPaddress = models.CharField(
        max_length=250,
        blank=True,
    )
    ad_description = models.CharField(
        max_length=250,
        blank=True,
    )
    position = models.CharField(
        max_length=250,
        blank=True,
    )
    department = models.CharField(
        max_length=250,
        blank=True,
    )
    comment = models.CharField(
        max_length=250,
        blank=True,
    )

    description = models.CharField(
        max_length=500,
        blank=True,
    )
    comments = models.TextField(blank=True)

    class Meta:
        ordering = ['name']

    def __str__(self):
        return self.name

    def get_absolute_url(self):
        return reverse('plugins:ptuevents:ptusers', args=[self.pk])


class PTWorkstations(NetBoxModel):

    name = models.CharField(
        max_length=100,
        unique=True
    )

    CN = models.CharField(
        max_length=250,
        unique=True,
        blank=False,
    )

    DistinguishedName = models.CharField(
        max_length=500,
        unique=True,
        blank=False,
    )

    ad_guid = models.CharField(
        max_length=250,
        blank=True,
    )

    ad_description = models.CharField(
        max_length=500,
        blank=True,
    )

    description = models.CharField(
        max_length=500,
        blank=True,
    )

    class Meta:
        ordering = ['name']

    def __str__(self):
        return self.name

    def get_absolute_url(self):
        return reverse('plugins:ptuevents:ptworkstations', args=[self.pk])




from attr import fields
from graphene import ObjectType
from netbox.graphql.types import NetBoxObjectType
from netbox.graphql.fields import ObjectField, ObjectListField
from . import filtersets
from .models import PTUEvent, PTUEventRelation, PTUEventAssignment, PTAppSystem, PTAppSystemAssignment


class PTUEventType(NetBoxObjectType):
    class Meta:
        model = PTUEvent
        fields = '__all__'


class PTUEventRelationType(NetBoxObjectType):
    class Meta:
        model = PTUEventRelation
        fields = '__all__'


class PTUEventAssingmentType(NetBoxObjectType):
    class Meta:
        model = PTUEventAssignment
        fields = '__all__'
        filterset_class = filtersets.PTUEventAssignmentFilterSet


class PTAppSystemType(NetBoxObjectType):
    class Meta:
        model = PTAppSystem
        fields = '__all__'


class PTAppSystemAssignmentType(NetBoxObjectType):
    class Meta:
        model = PTAppSystemAssignment
        fields = '__all__'
        filterset_class = filtersets.PTAppSystemAssignmentFilterSet


class Query(ObjectType):
    ptuevent = ObjectField(PTUEventType)
    ptuevent_list = ObjectListField(PTUEventType)
    relation = ObjectField(PTUEventRelationType)
    relation_list = ObjectListField(PTUEventRelationType)
    ptuevent_assignment = ObjectField(PTUEventAssingmentType)
    ptuevent_assignment_list = ObjectListField(PTUEventAssingmentType)
    ptapp_system = ObjectField(PTAppSystemType)
    ptapp_system_list = ObjectListField(PTAppSystemType)
    ptapp_system_assignment = ObjectField(PTAppSystemAssignmentType)
    ptapp_system_assignment_list = ObjectListField(PTAppSystemAssignmentType)


schema = Query



from extras.plugins import PluginConfig


class NetBoxPTUEventsConfig(PluginConfig):
    name = 'ptuevents'
    verbose_name = 'Unacceptable events and users and computers'
    description = 'Add events related fields to devices and virtual machines, adds application systems'
    version = '0.1.3.0026'
    base_url = 'ptuevents'
    author = 'Artur Shamsiev'
    author_email = 'me@z-lab.me'


config = NetBoxPTUEventsConfig



from extras.plugins import PluginTemplateExtension
from django.contrib.contenttypes.models import ContentType
from .models import PTUEventAssignment, PTAppSystemAssignment, PTAppSystem


class PTUEventVMPanel(PluginTemplateExtension):
    model = 'virtualization.virtualmachine'

    def right_page(self):
        vm = self.context['object']
        content_type_id = ContentType.objects.get_for_model(model=vm).id
        PTUEvent_ass = PTUEventAssignment.objects.filter(
            object_id=vm.id, content_type=content_type_id)
        PTUEvents = []
        for r in PTUEvent_ass:
            PTUEvents.append({
                'assignment_id': r.id,
                'name': r.ptuevent,
                'rel': r.relation.name
            })

        return self.render('ptuevents/ptuevent_panel.html', extra_context={
            'PTUEvents': PTUEvents
        })


# class PTUEventAppSystemPanel(PluginTemplateExtension):
#     model = 'PTUEvents.PTAppSystem'

#     def right_page(self):
#         app_system = self.context['object']
#         content_type_id = ContentType.objects.get_for_model(
#             model=app_system).id
#         PTUEvent_ass = PTUEventAssignment.objects.filter(
#             object_id=app_system.id, content_type=content_type_id)
#         PTUEvents = []
#         for r in PTUEvent_ass:
#             PTUEvents.append({
#                 'assignment_id': r.id,
#                 'name': r.PTUEvent,
#                 'rel': r.relation.name
#             })

#         return self.render('PTUEvents/PTUEvent_panel.html', extra_context={
#             'PTUEvents': PTUEvents
#         })


class PTUEventDevicePanel(PluginTemplateExtension):
    model = 'dcim.device'

    def right_page(self):
        device = self.context['object']
        content_type_id = ContentType.objects.get_for_model(model=device).id
        PTUEvent_ass = PTUEventAssignment.objects.filter(
            object_id=device.id, content_type=content_type_id)
        PTUEvents = []
        for r in PTUEvent_ass:
            PTUEvents.append({
                'assignment_id': r.id,
                'name': r.PTUEvent,
                'rel': r.relation.name
            })

        return self.render('ptuevents/ptuevent_panel.html', extra_context={
            'PTUEvents': PTUEvents
        })


class AppSystemVMPanel(PluginTemplateExtension):
    model = 'virtualization.virtualmachine'
    # model = 'dcim.device'

    def left_page(self):
        vm = self.context['object']
        content_type_id = ContentType.objects.get_for_model(model=vm).id
        app_systems = PTAppSystemAssignment.objects.filter(
            object_id=vm.id, content_type=content_type_id)
        # print(vars(AppSystem_ass))
        # print(AppSystem_ass)
        AppSystems = []
        for s in app_systems:
            AppSystems.append({
                'id': s.id,
                'app_system': s.app_system})
            # print(s.__dict__)

        # print(AppSystems)
        return self.render('ptuevents/appsystem_panel.html', extra_context={
            'app_systems': AppSystems
        })


class AppSystemDevicePanel(PluginTemplateExtension):
    model = 'dcim.device'

    def left_page(self):
        vm = self.context['object']
        content_type_id = ContentType.objects.get_for_model(model=vm).id
        app_systems = PTAppSystemAssignment.objects.filter(
            object_id=vm.id, content_type=content_type_id)
        AppSystems = []
        for s in app_systems:
            AppSystems.append({
                'id': s.id,
                'app_system': s.app_system})
            # print(s.__dict__)

        return self.render('ptuevents/appsystem_panel.html', extra_context={
            'app_systems': AppSystems
        })


template_extensions = [AppSystemVMPanel, AppSystemDevicePanel, PTUEventVMPanel,
                       PTUEventDevicePanel]
